// emacs: this is -*- c++ -*-
//
//   @file    mapCache.h        
//            
//            convenient cache type, allows any number of different values of 
//            observable for any give set of x1, x2, Q2, order
//  
//   Copyright (C) 2016 M.Sutton (sutt@cern.ch)    
//
//   $Id: mapCache.h, v0.0   Thu 14 Jul 2016 13:21:43 CEST sutt $


#ifndef  MAPCACHE_H
#define  MAPCACHE_H

#include <iostream>
#include <vector>
#include <map>

#include "imapCache.h"


extern "C" int currentcacheid_();

extern bool dbg;


struct mapCache : public imapCache< std::map<double, std::vector<double> > > {

public:

  mapCache( int nproc=0 ) : imapCache( nproc ) { 
    static bool first = true;
    if ( first ) { 
      std::cout << "mapCache(): nproc: " << nproc << std::endl;
      first = false;
    }
  } 

  virtual ~mapCache() { } 

  /// find the weight vector corresponding to this observable value
  /// or else add a new one to this cache

  virtual std::vector<double>& weights( double obs ) { 

    /// do we have a weight vector for this observable ?
    iterator itr =  this->find( obs );
    if ( itr!=this->end() ) return itr->second;

    /// no? then create one and return that 
    std::pair< iterator, bool > stat = this->insert( std::make_pair( obs, std::vector<double>( mnproc, 0 ) ) ) ;

    return stat.first->second;
  }

  

  /// have any of the kinematic variables changed ?
  
  virtual bool changed( double obs, int order, double x1, double x2, double Q2 ) {     
    return ( mx1!=x1 || mx2!=x2 || mQ2!=Q2 || morder!=order );
  }


  /// update the cached kinematics
    
  virtual void update( double obs, int order, double x1, double x2, double Q2 ) { 
    
    mfilled = true;

    mx1 = x1;
    mx2 = x2;
    mQ2 = Q2;

    morder = order;

    clear();
  }
  
};

inline std::ostream& operator<<( std::ostream& s, const mapCache& _p ) { 
  return s;
}


#endif  // MAPCACHE_H 










