// emacs: this is -*- c++ -*-
//
//   @file    vecCache.h        
//            
//            convenient cache type, allows any number of different values of 
//            observable for any give set of x1, x2, Q2, order
//  
//   Copyright (C) 2016 M.Sutton (sutt@cern.ch)    
//
//   $Id: vecCache.h, v0.0   Thu 14 Jul 2016 13:21:43 CEST sutt $


#ifndef  VECCACHE_H
#define  VECCACHE_H

#include <iostream>
#include <vector>
#include <map>

#include "nnlo_common.h"

#include "imapCache.h"

extern "C" int currentcacheid_();

extern bool dbg;


struct vecCache : public imapCache< std::vector< std::pair<double, std::vector<double> > > > {

public:

  vecCache( int nproc=0 ) : imapCache( nproc ) { 
    static bool first = true;
    if ( first ) { 
      std::cout << "vecCache(): nproc: " << nproc << std::endl;
      first = false;
    }
  } 

  virtual ~vecCache() { } 


  virtual std::vector<double>& weights( double obs ) { 

    /// do we have a weight vector for this observable ?
     
    unsigned int id = currentcacheid_();

    int iphase = iphase_.iphase;
    int Nphase = nphase_.Nphase;

    unsigned int myid = id*Nphase+(iphase-1);
    // int myid = id;

    //    if ( dbg ) 
    // std::cout << "cacheid " << id << "\tiphase " << iphase  << "\tmyid " << myid << "\tobs " << obs << "\tsize " << size() << std::endl;

    if ( myid<size() && at(myid).second.size()>0 ) {
      iterator itr = begin() + myid;
      // if ( dbg ) 
      // if ( obs!=itr->first )  std::cout << "cacheid " << id << "\tiphase " << iphase << "\tmyid " << myid << "\tobs " << obs << "\tsize " << size() << "\tdifferent obs: obs " << obs << "\tcache obs " << itr->first << std::endl;
      return itr->second;
    }


    if ( myid>=size() )  resize( myid+1 );
        
    /// No? then create one and return that 
    
    //    std::cout << "cahceid " << id << "\tiphase " << iphase  << "\tmyid " << myid << "\tobs " << obs << "\tsize " << size() << " (resize cache vec)" << std::endl;

    //  if ( dbg ) 
    //  std::cout << "cacheid " << currentcacheid_() << "\tiphase " << iphase << "\tNphase " << Nphase << "\tobs " << obs << "\t(" << mx1 << " " << mx2 << " " << mQ2 << ") ";    
    //   if ( dbg ) 
    // std::cout << "myid " << myid << "\tsize " << size() << "\tadd pair: obs " << obs << std::endl;

    at(myid) = std::make_pair( obs, std::vector<double>( mnproc, 0 ) );
    
    return at(myid).second;
  }
  
  
  
  /// have any of the kinematic variables changed ?
  
  virtual bool changed( double obs, int order, double x1, double x2, double Q2 ) {     
     //unsigned int id = currentcacheid_();    
    return ( mx1!=x1 || mx2!=x2 || mQ2!=Q2 || morder!=order ); // || ( id<size() && at(id).first!=obs ) );
  }


  /// update the cached kinematics
    
  virtual void update( double obs, int order, double x1, double x2, double Q2 ) { 
    
    mfilled = true;

    mx1 = x1;
    mx2 = x2;
    mQ2 = Q2;

    morder = order;

    clear();
  }
  
};

inline std::ostream& operator<<( std::ostream& s, const vecCache& _p ) { 
  return s;
}


#endif  // VECCACHE_H 











